@@ -1,8 +1,8 @@
 from textwrap import dedent
-from typing import List, Dict, Union, Tuple, Optional, Iterable
+from typing import List, Dict, Union, Tuple, Optional, Iterable, Type
 
 from . import CypherQuery
-from .base import camelcase, Varname, Statement, CypherVariable, CypherData, CypherVariableItem
+from .base import camelcase, Varname, Statement, CypherVariable, CypherData, CypherVariableItem, Collection
 
 
 def are_different(a: str, b: str) -> str:
@@ -31,7 +31,7 @@ def neo4j_dictionary(d: Union[dict, CypherVariable]) -> Tuple[Union[dict, Cypher
 
 
 def sanitise_variablename(v):
-    return f'{str(v).replace("$", "")}'
+    return f'`{str(v).replace("$", "")}`'
 
 
 def expand_to_cypher_dict(*collections: Union[Dict[str, CypherVariable], CypherVariable]) -> str:
@@ -111,23 +111,45 @@ class MatchRelationship(Statement):
 
 class MatchPatternNode(Statement):
     def __init__(self, labels: List[str], properties: Dict[str, Union[str, int, float, CypherVariable]],
-                 parents: List[CypherVariable], children: List[CypherVariable]):
+                 parents: List[CypherVariable], children: List[CypherVariable], exclude: List[CypherVariable]):
         self.labels = [camelcase(l) for l in labels]
         self.properties, inputs = neo4j_dictionary(properties)
-        self.parents = parents
-        self.children = children
+        self.parents = [p for p in parents if not isinstance(p, Collection)]
+        self.collection_parents = [p for p in parents if isinstance(p, Collection)]
+        self.children = [c for c in children if not isinstance(c, Collection)]
+        self.collection_children = [c for c in children if isinstance(c, Collection)]
+        self.exclude = [e for e in exclude if not isinstance(e, Collection)]
+        self.exclude_collection = [e for e in exclude if isinstance(e, Collection)]
         self.out = CypherVariable(labels[0])
-        super().__init__(inputs+parents+children, [self.out], [])
+        super().__init__(inputs+parents+children+exclude, [self.out], [])
 
     def to_cypher(self):
         labels = ':'.join(self.labels)
         match = f'({self.out}: {labels} {self.properties})'
-        wheres = ' AND '.join([f'({self.out})<--({p})' for p in self.parents] +
+        extras = ', '.join([f'({self.out})<--({p})' for p in self.parents] +
                             [f'({self.out})-->({c})' for c in self.children])
+        wheres = ' AND '.join([f"({self.out} <> {e})" for e in self.exclude] +
+                              [f"({self.out} <> {e})" for e in self.exclude_collection] +
+                              [f"all(p in {p} where exists( ({self.out})<--(p) ))" for p in self.collection_parents] +
+                              [f"all(c in {c} where exists( ({self.out})-->(c) ))" for c in self.collection_children])
+        if len(extras):
+            extras = f',{extras}'
         if len(wheres):
-            wheres = f'\nWHERE {wheres}'
-        return f'WITH * OPTIONAL MATCH {match}{wheres}'
+            wheres = f' WHERE {wheres}'
+        return f'WITH * OPTIONAL MATCH {match}{extras}{wheres}'
 
+class MatchBranchNode(Statement):
+    def __init__(self, *nodes_or_labels):
+        self.nodes_or_labels = nodes_or_labels
+        self.out = CypherVariable(nodes_or_labels)
+        inputs = [i for i in nodes_or_labels if isinstance(i, CypherVariable)]
+        outputs = [i  if isinstance(i, CypherVariable) else CypherVariable(i) for i in nodes_or_labels]
+        super().__init__(inputs, outputs, [])
+
+    def to_cypher(self):
+        nodes = [f'({o}:{n})' if isinstance(n, str) else f'({o})' for n, o in zip(self.nodes_or_labels, self.output_variables)]
+        path = '--'.join(map('{}'.format, nodes))
+        return f'WITH * OPTIONAL MATCH {path}'
 
 class PropertyOverlapError(Exception):
     pass
@@ -340,37 +362,71 @@ class MergeDependentNode(CollisionManager):
     @property
     def merge_statement(self):
         labels = ':'.join(map(str, self.labels))
-        relations = []
-        for i, (parent, reltype, relidentprop, dummyrelvar) in enumerate(zip(self.parents, self.reltypes, self.relidentproperties, self.dummyrelvars)):
-            rel = f'({parent})-[{dummyrelvar}:{reltype} {relidentprop}]->'
+        real_relations = []
+        temp_relations = []
+        test_relations = []
+        parent_list = []
+        for i, (parent, reltype, relidentprop, dummyrelvar, relvar) in enumerate(zip(self.parents, self.reltypes, self.relidentproperties, self.dummyrelvars, self.relvars)):
             if i == 0:
                 child = f'({self.dummy}: {labels} {self.identproperties})'
             else:
                 child = f'({self.dummy})'
-            relations.append(rel + child)
-        optional_match = 'OPTIONAL MATCH ' + ',\n'.join(relations)
-        create = '\n'.join(['MERGE ' + r for r in relations])
-        for dummy, real in zip(self.dummyrelvars, self.relvars):
-            create = create.replace(f'[{dummy}:', f'[{real}:')
-        create = create.replace(f'{self.dummy}', f'{self.out}')
+            rel = f'({parent})-[{dummyrelvar}:{reltype} {relidentprop}]->'
+            real_rel = f'({parent})-[{relvar}:{reltype} {relidentprop}]->'
+            real_child = f'({self.out})'
+            real_relations.append(real_rel + real_child)
+            temp_relations.append(rel + '(temp)')
+            test_relations.append(rel + child)
+            parent_list.append(f"{parent}")
+        dct = expand_to_cypher_dict(self.dummy, self.propvar, self.identproperties, *self.parents + self.relidentproperties + self.dummyrelvars)
+        aliases = expand_to_cypher_alias(self.identproperties, *self.parents + self.relidentproperties)
+        variables = set()
+        for p in self.parents:
+            variables.add(p)
+        for v in self.identproperties.values():
+            try:
+                v = v.parent
+            except AttributeError:
+                pass
+            if not isinstance(v, CypherData):
+                variables.add(v)
+        for rel in self.relidentproperties:
+            for v in rel.values():
+                try:
+                    v = v.parent
+                except AttributeError:
+                    pass
+                if not isinstance(v, CypherData):
+                    variables.add(v)
+        merge_temp_relations = '\n'.join([f'MERGE {r}' for r in temp_relations])
+        merge_final_temp_relation = f"MERGE (temp)-[:TemporaryMerge]->({self.out}: {labels} {self.identproperties})"
+        merge_real_relations = '\n'.join([f'MERGE {r}' for r in real_relations])
         on_create_rel_returns = ', '.join([f'{relvar}' for relvar in self.relvars])
-        on_match_rel_returns = ', '.join([f'{dummy} as {real}' for dummy, real in zip(self.dummyrelvars, self.relvars)])
+        on_match_rel_returns = ', '.join([f'${dummy}[0] as {real}' for dummy, real in zip(self.dummyrelvars, self.relvars)])
         rel_expansion = expand_to_cypher_alias(self.out, *self.relvars, prefix=f'{self.child_holder}.')
-        aliases = expand_to_cypher_alias(self.identproperties, *self.parents+self.relidentproperties+self.dummyrelvars)
-        dct = expand_to_cypher_dict(self.dummy, self.propvar, self.identproperties, *self.parents+self.relidentproperties+self.dummyrelvars)
-        query = f"""
-        CALL apoc.lock.nodes({self.parents}) // let's lock ahead this time
-        {optional_match}
-        call apoc.do.when({self.dummy} IS NULL, 
-                "WITH {aliases} 
-                {create}
-                SET {self.out} += ${self.propvar}
-                RETURN {self.out}, {on_create_rel_returns}",   // created
-            "RETURN ${self.dummy} as {self.out}, {on_match_rel_returns}",  // matched 
-            {{ {dct} }}) yield value as {self.child_holder}
-        WITH *, {rel_expansion}
+        optional_match = f'OPTIONAL MATCH {test_relations[0]}'
+        matches = '\n'.join([f'MATCH {t}' for t in test_relations])
+        rel_collection = ', '.join([f'collect({drel}) as {drel}' for rel, drel in zip(self.relvars, self.dummyrelvars)])
+        collection = f'CALL {{ WITH {", ".join(map(str, variables))}\n' \
+                     f'{optional_match}\n' \
+                     f'{matches}\n' \
+                     f'RETURN collect({self.dummy}) as {self.dummy}, {rel_collection}\n' \
+                     f'}}'
+        condition = f"size({self.dummy}) = 0"
+        iftrue = f"""
+        WITH {aliases}
+        MERGE (temp: TemporaryMerge {{id: $time0}})
+        {merge_temp_relations}
+        {merge_final_temp_relation}
+        {merge_real_relations}
+        DETACH DELETE temp
+        SET {self.out} += ${self.propvar}
+        RETURN {self.out}, {on_create_rel_returns}
         """
-        return dedent(query)
+        iffalse = f"RETURN ${self.dummy}[0] as {self.out}, {on_match_rel_returns}"
+        when = f'CALL apoc.do.when({condition}, "{iftrue}", "{iffalse}", {{ {dct}, time0:time0}}) yield value as {self.child_holder}'
+        when += f"\n WITH *, {rel_expansion}"
+        return dedent(f"CALL apoc.lock.nodes({self.parents})\n{collection}\n{when}")
 
     @property
     def on_match(self):  # remember, we are in a call context
@@ -419,7 +475,7 @@ class MergeDependentNode(CollisionManager):
         aliases = expand_to_cypher_alias(self.out, self.propvar, *self.relvars+self.relpropsvars)
         return dedent(f"""
         // post merge
-        call apoc.do.when({self.dummy} IS NULL,
+        call apoc.do.when(size({self.dummy}) = 0,
         "WITH {aliases}\n{self.on_create}\n RETURN $time0",
         "WITH {aliases}\n{self.on_match}\n RETURN $time0",
         {{ {dct} }}) yield value as {self.unnamed}
@@ -479,7 +535,8 @@ def match_relationship(parent, child, reltype, properties, optional=False):
 
 
 def match_pattern_node(labels: List[str], properties: Dict[str, Union[str, int, float, CypherVariable]] = None,
-                       parents: List[CypherVariable] = None, children: List[CypherVariable] = None):
+                       parents: List[CypherVariable] = None, children: List[CypherVariable] = None,
+                       exclude: List[CypherVariable] = None):
     query = CypherQuery.get_context()  # type: CypherQuery
     if properties is None:
         properties = {}
@@ -487,11 +544,20 @@ def match_pattern_node(labels: List[str], properties: Dict[str, Union[str, int,
         parents = []
     if children is None:
         children = []
-    statement = MatchPatternNode(labels, properties, parents, children)
+    if exclude is None:
+        exclude = []
+    statement = MatchPatternNode(labels, properties, parents, children, exclude)
     query.add_statement(statement)
     return statement.out
 
 
+def match_branch_node(*nodes_or_labels: Union[str, Type[CypherVariable]]):
+    query = CypherQuery.get_context()  # type: CypherQuery
+    statement = MatchBranchNode(*nodes_or_labels)
+    query.add_statement(statement)
+    return statement.output_variables
+
+
 def merge_single_node(labels, identproperties, properties, collision_manager='track&flag'):
     query = CypherQuery.get_context()  # type: CypherQuery
     statement = MergeNode(labels, identproperties, properties, collision_manager)
